@@ -2,14 +2,14 @@ package Starlet;
 
 use 5.008_001;
 
-our $VERSION = '0.27_01';
+our $VERSION = '0.05';
 
 1;
 __END__
 
 =head1 NAME
 
-Starlet - a simple, high-performance PSGI/Plack HTTP server
+Starlet
 
 =head1 SYNOPSIS
 
@@ -21,7 +21,7 @@ Starlet - a simple, high-performance PSGI/Plack HTTP server
 
 =head1 DESCRIPTION
 
-Starlet is a standalone HTTP/1.1 web server, formerly known as L<Plack::Server::Standalone::Prefork> and L<Plack::Server::Standalone::Prefork::Server::Starter>.
+Starlet is a standalone HTTP/1.0 server formerly known as L<Plack::Server::Standalone::Prefork> and L<Plack::Server::Standalone::Prefork::Server::Starter>.
 
 The server supports following features, and is suitable for running HTTP application servers behind a reverse proxy.
 
@@ -35,7 +35,7 @@ The server supports following features, and is suitable for running HTTP applica
 
 In addition to the options supported by L<plackup>, Starlet accepts following options(s).
 
-=head2 --max-workers=#
+=head2 --num-workers=#
 
 number of worker processes (default: 10)
 
@@ -55,17 +55,9 @@ max. number of requests allowed per single persistent connection.  If set to one
 
 max. number of requests to be handled before a worker process exits (default: 100)
 
-=head2 --min-reqs-per-child=#
-
-if set, randomizes the number of requests handled by a single worker process between the value and that supplied by C<--max-reqs-per-chlid> (default: none)
-
-=head2 --spawn-interval=#
-
-if set, worker processes will not be spawned more than once than every given seconds.  Also, when SIGHUP is being received, no more than one worker processes will be collected every given seconds.  This feature is useful for doing a "slow-restart".  See http://blog.kazuhooku.com/2011/04/web-serverstarter-parallelprefork.html for more information. (default: none)
-
 =head1 NOTES
 
-L<Starlet> is designed and implemented to be simple, secure and fast, especially for running as an HTTP application server running behind a reverse proxy.  It only depends on a minimal number of well-designed (and well-focused) modules.
+If you are looking for a standalone preforking HTTP server, then you should really look at L<Starman>.  However if your all want is a simple HTTP server that runs behind a reverse proxy, this good old module still does what it used to.
 
 =head1 SEE ALSO
 
@@ -79,10 +71,6 @@ Kazuho Oku
 
 miyagawa
 
-kazeburo
-
-Tomohiro Takezawa
-
 =head1 LICENSE
 
 This program is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
